/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package cz.soma.portal.kooperace.controller;

import cz.soma.portal.kooperace.dao.CommentDao;
import cz.soma.portal.kooperace.dao.DemandDao;
import cz.soma.portal.kooperace.dao.OrderDao;
import cz.soma.portal.kooperace.dao.UserDao;
import cz.soma.portal.kooperace.model.CalendarSettings;
import cz.soma.portal.kooperace.model.Comment;
import cz.soma.portal.kooperace.model.Demand;
import cz.soma.portal.kooperace.model.Order;
import cz.soma.portal.kooperace.model.User;
import cz.soma.portal.kooperace.utils.PortalUtils;
import cz.soma.portal.kooperace.utils.SecurityUtils;
import java.security.Principal;
import java.text.ParseException;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.servlet.ModelAndView;

/**
 *
 * @author berant
 */
@Controller
public class MainController {

    @Autowired
    DemandDao demandDao;
    @Autowired
    OrderDao orderDao;
    @Autowired
    CommentDao commentDao;
    @Autowired
    UserDao userDao;

    @RequestMapping("/index")
    public ModelAndView defaultPage(@RequestParam(required = false, value = "error") String error) {

        ModelAndView model = new ModelAndView();
        model.setViewName("index");

        return model;
    }
    
    @RequestMapping("/news")
    public ModelAndView getNewsFrame() {

        ModelAndView model = new ModelAndView("news");
        String news = PortalUtils.getNewsData();
        model.addObject("news", news);
        return model;
    }

    @RequestMapping("/contact")
    public ModelAndView contact() {

        ModelAndView model = new ModelAndView();
        model.setViewName("contact");

        return model;
    }

    @RequestMapping("/account")
    public ModelAndView account() {

        ModelAndView model = new ModelAndView("account");
        User u = SecurityUtils.getUserById(userDao);
        model.addObject("user", u);

        return model;
    }

    @RequestMapping("/dashboard")
    public ModelAndView dashboard(@RequestParam(required = false, value = "month_from") String monthFromStr,
            @RequestParam(required = false, value = "year_from") String yearFromStr,
            @RequestParam(required = false, value = "month_to") String monthToStr,
            @RequestParam(required = false, value = "year_to") String yearToStr) {

        int monthFrom = PortalUtils.strToIntDef(monthFromStr);
        int yearFrom = PortalUtils.strToIntDef(yearFromStr);
        int monthTo = PortalUtils.strToIntDef(monthToStr);
        int yearTo = PortalUtils.strToIntDef(yearToStr);

        ModelAndView model = new ModelAndView();
        model.setViewName("dashboard");

        CalendarSettings cs;
        User u = SecurityUtils.getUserById(userDao);
        model.addObject("user", u);
        try {
            //validace udaju
            if (!PortalUtils.isValidMonthAndYearValues(monthFrom, yearFrom)) {
                monthFrom = PortalUtils.getCurrentDayField(Calendar.MONTH) + 1; //Pozor Calendar pocita mesice od O
                yearFrom = PortalUtils.getCurrentDayField(Calendar.YEAR);
            }
            if (!PortalUtils.isValidMonthAndYearValues(monthTo, yearTo) || PortalUtils.firstDayOfMonth(monthTo, yearTo).compareTo(PortalUtils.firstDayOfMonth(monthFrom, yearFrom)) < 0) {
                monthTo = PortalUtils.getCurrentDayField(Calendar.MONTH, true) + 1;
                yearTo = PortalUtils.getCurrentDayField(Calendar.YEAR, true);
            }
            Date dateFrom = PortalUtils.firstDayOfMonth(monthFrom, yearFrom);
            Date dateTo = PortalUtils.lastDayOfMonth(monthTo, yearTo);

            cs = new CalendarSettings(dateFrom, dateTo, u, demandDao, orderDao);
            model.addObject("cs", cs);
            //vypocitane hodnoty mesic a rok
            model.addObject("date_from", monthFrom);
            model.addObject("year_from", yearFrom);
            model.addObject("date_to", monthTo);
            model.addObject("year_to", yearTo);
        } catch (ParseException ex) {
            Logger.getLogger(MainController.class.getName()).log(Level.SEVERE, null, ex);
        }

        return model;
    }

    @RequestMapping("/demand")
    public ModelAndView demand() {

        ModelAndView model = new ModelAndView("demand");
        List<Demand> demands = demandDao.getDemands(SecurityUtils.getUserId());
        model.addObject("demands", demands);
        User u = SecurityUtils.getUserById(userDao);
        model.addObject("user", u);
        return model;
    }

    @RequestMapping("/demand-detail")
    public ModelAndView demandDetail(@RequestParam(name = "demand") int demand,
            @RequestParam(required = false, name = "success") String success) {

        ModelAndView model = new ModelAndView("demand-detail");
        User u = SecurityUtils.getUserById(userDao);
        if (demand > 0) {
            Demand d = demandDao.getDemand(SecurityUtils.getUserId(), demand);
            model.addObject("demand", d);
            //stavajici komentare
            List<Comment> c = commentDao.getComment(u.getUserId(), d.getCislo(), Comment.STAV_OBJ_POPTAVKA);
            model.addObject("comments", c);
        }

        //novy komentar pripraveny pro vlozeni
        Comment newComment = new Comment(Comment.STAV_OBJ_POPTAVKA, u.getName());
        model.addObject("comment", newComment);
        model.addObject("user", u);
        model.addObject("success", success);
        return model;
    }

    @RequestMapping("/order-detail")
    public ModelAndView orderDetail(@RequestParam(name = "order") int order,
            @RequestParam(required = false, name = "success") String success) {
        User u = SecurityUtils.getUserById(userDao);

        ModelAndView model = new ModelAndView("order-detail");
        Order o = orderDao.getOrder(u.getUserId(), order);
        model.addObject("order", o);
        if (o != null) {
            //stavajici komentare
            List<Comment> c = commentDao.getComment(u.getUserId(), o.getCislo(), Comment.STAV_OBJ_OBJEDNAVKA);
            model.addObject("comments", c);
        }

        //novy komentar pripraveny pro vlozeni
        Comment newComment = new Comment(Comment.STAV_OBJ_OBJEDNAVKA, u.getName());
        model.addObject("comment", newComment);
        model.addObject("user", u);
        model.addObject("success", success);
        return model;
    }

    @RequestMapping("/contact-logged")
    public ModelAndView contactLogged() {

        ModelAndView model = new ModelAndView();
        model.setViewName("contact-logged");
        User u = SecurityUtils.getUserById(userDao);
        model.addObject("user", u);

        return model;
    }

    @RequestMapping("/order")
    public ModelAndView order() {

        ModelAndView model = new ModelAndView("order");
        List<Order> orders = orderDao.getOrders(SecurityUtils.getUserId());
        model.addObject("orders", orders);
        User u = SecurityUtils.getUserById(userDao);
        model.addObject("user", u);
        return model;
    }

    @RequestMapping("/403")
    public ModelAndView accessDeniedPage(Principal user) {
        ModelAndView model = new ModelAndView("index");
        model.addObject("uzivatel", user != null? user.getName() : "");

        return model;
    }

}
