/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package cz.soma.portal.kooperace.controller;

import cz.soma.infosys.components.SDatabase;
import cz.soma.portal.kooperace.dao.DemandDao;
import cz.soma.portal.kooperace.dao.OrderDao;
import cz.soma.portal.kooperace.dao.UserDao;
import cz.soma.portal.kooperace.model.Demand;
import cz.soma.portal.kooperace.model.Item;
import cz.soma.portal.kooperace.model.Order;
import cz.soma.portal.kooperace.model.User;
import cz.soma.portal.kooperace.utils.DataUtils;
import cz.soma.portal.kooperace.utils.PortalUtils;
import cz.soma.portal.kooperace.utils.SecurityUtils;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.openide.util.Exceptions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.util.FileCopyUtils;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;

/**
 *
 * @author berant
 */
@Controller
public class PdfController {

    @Autowired
    DemandDao demandDao;
    @Autowired
    OrderDao orderDao;
    @Autowired
    SDatabase database;
    @Autowired
    UserDao userDao;

    @RequestMapping("/downloadOrder")
    public void downloadOrder(@RequestParam(required = true, value = "order") int orderNumber, HttpServletResponse response) {

        Order order = orderDao.getOrder(SecurityUtils.getUserId(), orderNumber);
        if (order == null) {
            try {
                response.sendError(404, "Access denied");
            } catch (IOException ex) {
                Exceptions.printStackTrace(ex);
            }
            return;
        }

        Map<String, Object> defaultParams = new HashMap<String, Object>();
        defaultParams.put("objednavka", orderNumber);
        String reportName = (order.getTyp().equals("K") ? "MTZ_OBJEDNAVKA_KOOP_" : "MTZ_OBJEDNAVKA_") + PortalUtils.getDefaultLocaleOfReport();

        DataUtils.downloadReport(reportName, defaultParams, "Objednavka-" + orderNumber, response, database);
    }

    @RequestMapping("/downloadDemand")
    public void downloadDemand(@RequestParam(required = true, value = "demand") int demandNumber, HttpServletResponse response) {
        Demand demand = demandDao.getDemand(SecurityUtils.getUserId(), demandNumber);
        if (demand == null) {
            try {
                response.sendError(404, "Access denied");
            } catch (IOException ex) {
                Exceptions.printStackTrace(ex);
            }
            return;
        }

        Map<String, Object> defaultParams = new HashMap<String, Object>();
        defaultParams.put("objednavka", demandNumber);
        String reportName = (demand.getTyp().equals("K") ? "MTZ_POPTAVKA_KOOP_" : "MTZ_POPTAVKA_") + PortalUtils.getDefaultLocaleOfReport();

        DataUtils.downloadReport(reportName, defaultParams, "Poptavka-" + demandNumber, response, database);
    }

    @RequestMapping("/downloadData/{orderNumber}")
    public void downloadData(@PathVariable(value = "orderNumber") Integer orderNumber,
            HttpServletRequest request, HttpServletResponse response) {
        OutputStream o = null;
        InputStream is = null;
        File zipFile = null;
        try {
            User u = SecurityUtils.getUserById(userDao);
            //cisloObjednavky musi byt vyplneno
            if (orderNumber == null || orderNumber <= 0) {
                throw new Exception("Číslo objednávky nenalezeno!");
            }
            //zjistim si vsechny polozky dane objednavky
            List<Item> items = orderDao.getOrder(u.getUserId(), orderNumber).getItems();
            List<String> dataList = new ArrayList<String>();
            for (Item i : items) {
                if (request.getParameter(String.format("polozka[%d]", i.getPolozka())) != null) {
                    String dxf = i.getDataDxf();
                    String step = i.getDataStep();
                    String iges = i.getDataIges();
                    if (!dxf.isEmpty()) {
                        dataList.add(dxf);
                    }
                    if (!step.isEmpty()) {
                        dataList.add(step);
                    }
                    if (!iges.isEmpty()) {
                        dataList.add(iges);
                    }
                }
            }
            if (dataList.isEmpty()) {
                throw new Exception(String.format("Na vybrané objednávce %d nebyli zaškrtnuty žádné položky obsahující 3D data!", orderNumber));
            }
            Calendar c = Calendar.getInstance();
            File dir = new File(String.format("/tmp/%s-%d%d%d/",
                    u.getUserId(),
                    c.get(Calendar.HOUR),
                    c.get(Calendar.MINUTE),
                    c.get(Calendar.SECOND)));
            dir.delete();
            dir.mkdir();
            zipFile = DataUtils.getDataVykresZArchivu(dataList, dir, u.getCompanyName());
            response.setContentType("application/zip");
            o = response.getOutputStream();
            is = new FileInputStream(zipFile);
            FileCopyUtils.copy(is, o);

            response.flushBuffer();
        } catch (Exception ex) {
            
            try {
                response.sendError(404, ex.getMessage());
            } catch (IOException ex1) {
                Exceptions.printStackTrace(ex1);
            }

        } finally {
            try {
                if (o != null) {
                    o.close();
                }
            } catch (IOException ex) {
                Exceptions.printStackTrace(ex);
            }

            try {
                if (is != null) {
                    is.close();
                }
            } catch (IOException ex) {
                Logger.getLogger(PdfController.class.getName()).log(Level.SEVERE, null, ex);
            }
            if (zipFile != null) {
                zipFile.delete();
            }
        }
    }

    @RequestMapping("/downloadVykres/{archiv_vykres}")
    public void downloadVykres(@PathVariable(value = "archiv_vykres") String vykres,
            @RequestParam(required = true, value = "order") Integer orderNumber,
            HttpServletResponse response) {

        try {
            //vykres musi byt vyplnen
            if (vykres == null || vykres.isEmpty() || orderNumber == null || orderNumber <= 0) {
                throw new Exception("Výkres nebyl nalezen.");
            }
            //na dane objednavce musi pozadovany vykres byt mezi polozkami (tzn. uzivatel ma pristup k vykresu)
            List<Item> items = orderDao.getOrder(SecurityUtils.getUserId(), orderNumber).getItems();
            boolean nalezen = false;
            for (Item i : items) {
                if (i.getArchivVykres().equals(vykres)) {
                    nalezen = true;
                    break;
                }
            }
            if (!nalezen) {
                throw new Exception(String.format("K požadovanému výkresu nemáte přístup, nenachází se na objednávce %d", orderNumber));
            }

            DataUtils.downloadVykres(vykres, SecurityUtils.getUserId(), response);

        } catch (Exception ex) {

            try {
                response.sendError(404, ex.getMessage());
            } catch (IOException ex1) {
                Exceptions.printStackTrace(ex1);
            }
        } 
    }
}
