/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package cz.soma.portal.kooperace.dao;

import cz.soma.portal.kooperace.model.Demand;
import cz.soma.portal.kooperace.model.Item;
import cz.soma.portal.kooperace.utils.PortalUtils;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import org.hibernate.HibernateException;
import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.Transaction;
import org.springframework.transaction.annotation.Transactional;

/**
 *
 * @author berant
 */
@Transactional
public class DemandDaoImpl implements DemandDao {

    private SessionFactory sessionFactory;

    public DemandDaoImpl(SessionFactory sessionFactory) {
        this.sessionFactory = sessionFactory;
    }

    @Override
    public List<Demand> getDemands(String id) {

        Session session = sessionFactory.getCurrentSession().getSessionFactory().openSession();

        List<Demand> demands = null;
        Transaction trans = null;
        try {
            trans = session.getTransaction();
            trans.begin();
            demands = (List<Demand>) session.createQuery(
                    String.format("from Demand d where d.uzivatel = %s",
                            PortalUtils.getQuotedStr(id))).list();
            PortalUtils.commitTransaction(trans);
        } catch (HibernateException e) {
            e.printStackTrace();
            PortalUtils.rollbackTransaction(trans);
        } finally {
            PortalUtils.closeSession(session);
        }

        return demands;
    }

    @Override
    public Demand getDemand(String id, int demand) {
        Session session = sessionFactory.getCurrentSession().getSessionFactory().openSession();

        Demand d = null;
        Transaction trans = null;
        try {
            trans = session.getTransaction();
            trans.begin();
            d = (Demand) session.createQuery(String.format("from Demand d where d.cislo = %d and d.uzivatel = %s", 
                    demand, PortalUtils.getQuotedStr(id))).uniqueResult();
            PortalUtils.commitTransaction(trans);
        } catch (HibernateException e) {
            e.printStackTrace();
            PortalUtils.rollbackTransaction(trans);
        } finally {
            PortalUtils.closeSession(session);
        }

        return d;
    }

    @Override
    public List<Item> getDemandItems(String id, int demand) {
        List<Item> items = null;
        Session session = sessionFactory.getCurrentSession().getSessionFactory().openSession();
        Transaction trans = null;
        try {
            trans = session.getTransaction();
            trans.begin();
            items = (List<Item>) session.createQuery(
                    String.format("from Item i where i.uzivatel = %s and i.objednavka = %d",
                            PortalUtils.getQuotedStr(id), demand)).list();

            PortalUtils.commitTransaction(trans);
        } catch (HibernateException e) {
            e.printStackTrace();
            PortalUtils.rollbackTransaction(trans);
        } finally {
            PortalUtils.closeSession(session);
        }

        return items;
    }

    @Override
    public Map<Date, Integer> getDemandsCountBetweenDate(String id, Date dateFrom, Date dateTo, boolean confirmed) {
        String sql = String.format("select D.TERMIN_POZADOVANY, count(D.OBJEDNAVKA)\n"
                + "from KOO_DEMANDS D\n"
                + "where D.SUPPORT_USERNAME = %s and D.TERMIN_POZADOVANY between %s and %s"
                + "    and D.DATUM_PRIJETI is %s null\n"
                + "group by 1",
                PortalUtils.getQuotedStr(id),
                PortalUtils.getParametrizedDate(dateFrom, true),
                PortalUtils.getParametrizedDate(dateTo, true), 
                confirmed? "not" : "");

        Session session = sessionFactory.getCurrentSession().getSessionFactory().openSession();
        Map<Date, Integer> map = new HashMap<Date, Integer>();
        Transaction trans = null;
        try {
            trans = session.getTransaction();
            trans.begin();
            List<Object[]> res = session.createSQLQuery(sql).list();
            for (Object[] re : res) {
                map.put((Date) re[0], (Integer) re[1]);
            }
            PortalUtils.commitTransaction(trans);
        } catch (HibernateException e) {
            e.printStackTrace();
            PortalUtils.rollbackTransaction(trans);
        } finally {
            PortalUtils.closeSession(session);
        }

        return map;
    }

    @Override
    public boolean confirmDemand(String id, int demand, Date confirmDate) {
        boolean retVal = false;
        String sql = "update Demand set datumPrijeti = :datum where cislo = :cislo and uzivatel = :id";
        Session session = sessionFactory.getCurrentSession().getSessionFactory().openSession();
        Transaction trans = null;
        try {
            trans = session.getTransaction();
            trans.begin();
            Query q = session.createQuery(sql);
            q.setDate("datum", confirmDate);
            q.setInteger("cislo", demand);
            q.setString("id", id);
            
            retVal = q.executeUpdate() == 1;

            PortalUtils.commitTransaction(trans);
        } catch (HibernateException e) {
            e.printStackTrace();
            PortalUtils.rollbackTransaction(trans);
        } finally {
            PortalUtils.closeSession(session);
        }

        return retVal;
    }
}
