/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package cz.soma.portal.kooperace.dao;

import cz.soma.portal.kooperace.model.Item;
import cz.soma.portal.kooperace.model.Order;
import cz.soma.portal.kooperace.utils.PortalUtils;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import org.hibernate.HibernateException;
import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.Transaction;
import org.springframework.transaction.annotation.Transactional;

@Transactional
public class OrderDaoImpl implements OrderDao {

    private SessionFactory sessionFactory;

    public OrderDaoImpl(SessionFactory sessionFactory) {
        this.sessionFactory = sessionFactory;
    }

    @Override
    public List<Order> getOrders(String id) {
        Session session = sessionFactory.getCurrentSession().getSessionFactory().openSession();

        List<Order> orders = null;
        Transaction trans = null;
        try {
            trans = session.getTransaction();
            trans.begin();
            orders = (List<Order>) session.createQuery(
                    String.format("from Order o where o.uzivatel = %s",
                            PortalUtils.getQuotedStr(id))).list();
            PortalUtils.commitTransaction(trans);
        } catch (HibernateException e) {
            e.printStackTrace();
            PortalUtils.rollbackTransaction(trans);
        } finally {
            PortalUtils.closeSession(session);
        }

        return orders;
    }

    @Override
    public Order getOrder(String id, int order) {
        Session session = sessionFactory.getCurrentSession().getSessionFactory().openSession();

        Order d = null;
        Transaction trans = null;
        try {
            trans = session.getTransaction();
            trans.begin();
            d = (Order) session.createQuery(String.format("from Order o where o.uzivatel = %s and o.cislo = %d",
                    PortalUtils.getQuotedStr(id), order)).uniqueResult();
            PortalUtils.commitTransaction(trans);
        } catch (HibernateException e) {
            e.printStackTrace();
            PortalUtils.rollbackTransaction(trans);
        } finally {
            PortalUtils.closeSession(session);
        }

        return d;
    }

    @Override
    public List<Item> getOrderItems(String id, int order) {
        throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
    }

    @Override
    public Map<Date, Integer> getOrdersCountBetweenDate(String id, Date dateFrom, Date dateTo, boolean confirmed) {
        String sql = String.format("select O.TERMIN, count(O.OBJEDNAVKA)\n"
                + "from KOO_ORDERS O\n"
                + "where O.SUPPORT_USERNAME = %s and O.TERMIN between %s and %s"
                + " and O.DATUM_POTVRZENI is %s null\n"
                + "group by 1",
                PortalUtils.getQuotedStr(id),
                PortalUtils.getParametrizedDate(dateFrom, true),
                PortalUtils.getParametrizedDate(dateTo, true),
                confirmed ? "not" : "");

        Session session = sessionFactory.getCurrentSession().getSessionFactory().openSession();
        Map<Date, Integer> map = new HashMap<Date, Integer>();
        Transaction trans = null;
        try {
            trans = session.getTransaction();
            trans.begin();
            List<Object[]> res = session.createSQLQuery(sql).list();
            for (Object[] re : res) {
                map.put((Date) re[0], (Integer) re[1]);
            }
            PortalUtils.commitTransaction(trans);
        } catch (HibernateException e) {
            e.printStackTrace();
            PortalUtils.rollbackTransaction(trans);
        } finally {
            PortalUtils.closeSession(session);
        }

        return map;
    }

    @Override
    public boolean confirmOrder(String id, int order, Date date) {
        boolean retVal = false;
        String sql = "update Order set datumPotvrzeni = :datum where cislo = :cislo and uzivatel = :id";
        Session session = sessionFactory.getCurrentSession().getSessionFactory().openSession();
        Transaction trans = null;
        try {
            trans = session.getTransaction();
            trans.begin();
            Query q = session.createQuery(sql);
            q.setDate("datum", date);
            q.setInteger("cislo", order);
            q.setString("id", id);

            retVal = q.executeUpdate() == 1;

            PortalUtils.commitTransaction(trans);
        } catch (HibernateException e) {
            e.printStackTrace();
            PortalUtils.rollbackTransaction(trans);
        } finally {
            PortalUtils.closeSession(session);
        }

        return retVal;
    }

    @Override
    public boolean changeTermin(String id, int order, int item, Date newTermin) {
        boolean retVal = false;
        String sql = "update Item set terminPotvrzeny = :datum where cislo = :order and polozka = :polozka and uzivatel = :id";
        Session session = sessionFactory.getCurrentSession().getSessionFactory().openSession();
        Transaction trans = null;
        try {
            trans = session.getTransaction();
            trans.begin();
            Query q = session.createQuery(sql);
            q.setDate("datum", newTermin);
            q.setInteger("order", order);
            q.setInteger("polozka", item);
            q.setString("id", id);

            retVal = q.executeUpdate() == 1;

            PortalUtils.commitTransaction(trans);
        } catch (HibernateException e) {
            e.printStackTrace();
            PortalUtils.rollbackTransaction(trans);
        } finally {
            PortalUtils.closeSession(session);
        }

        return retVal;
    }
}
