/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package cz.soma.portal.kooperace.model;

import cz.soma.portal.kooperace.dao.DemandDao;
import cz.soma.portal.kooperace.dao.OrderDao;
import cz.soma.portal.kooperace.utils.PortalUtils;
import cz.soma.portal.kooperace.utils.SecurityUtils;
import java.util.Calendar;
import java.util.Date;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

/**
 *
 * @author berant
 */
public class CalendarSettings {

    public static final int WEEK = 7; //pocet dnu v tydnu

    private Date dateFrom;
    private Date dateTo;

    private int rowCount = 5;
    private int colCount = 0;
    private int monthCount = 0;

    List<CalendarDay> days = null;
    
    private int demandUnconfirmed = 0;
    private int orderUnconfirmed = 0;

    public CalendarSettings(Date start, Date end, User u, DemandDao demandDao, OrderDao orderDao) {
        this.monthCount = getMonthCount(start, end);
        this.dateFrom = getFirstMonday(start);
        this.dateTo = getLastSunday(end);
        
        colCount = WEEK * monthCount;

        days = new LinkedList<CalendarDay>();
        //pocetMesicu = pocetSloupcu / 7;
        Calendar cal = Calendar.getInstance();
        cal.setTime(dateFrom);
        for (int m = 1; m <= monthCount; m++) {
            for (int w = 1; w <= rowCount; w++) {
                for (int d = 1; d <= WEEK; d++) {
                    days.add(new CalendarDay(w, m*d, cal.getTime(), cal.getTime().compareTo(start) >= 0 && cal.getTime().compareTo(end) <= 0));
                    cal.add(Calendar.DAY_OF_YEAR, 1);
                }
            }
        }
        
        //do kazdeho dne nastavim pocty poptavek a objednavek + pocitam si celkove pocty nepotvrzenych popt + obj
        String userID = u == null? SecurityUtils.getUserId() : u.getUserId();
        Map<Date, Integer> demandsConfirmed = demandDao.getDemandsCountBetweenDate(userID, dateFrom, dateTo, true);
        Map<Date, Integer> demandsUnconfirmed = demandDao.getDemandsCountBetweenDate(userID, dateFrom, dateTo, false);
        Map<Date, Integer> ordersConfirmed = orderDao.getOrdersCountBetweenDate(userID, dateFrom, dateTo, true);
        Map<Date, Integer> ordersUnconfirmed = orderDao.getOrdersCountBetweenDate(userID, dateFrom, dateTo, false);
        
        for (CalendarDay day : days) {
            /*TB20160129 funkcni pouze v Java 8 & Tomcat 8
            Integer demC = demandsConfirmed.getOrDefault(day.getDatum(), 0);
            Integer demU = demandsUnconfirmed.getOrDefault(day.getDatum(), 0);
            Integer ordC = ordersConfirmed.getOrDefault(day.getDatum(), 0);
            Integer ordU = ordersUnconfirmed.getOrDefault(day.getDatum(), 0);*/
            Integer demC = PortalUtils.notNullInt(demandsConfirmed.get(day.getDatum()));
            Integer demU = PortalUtils.notNullInt(demandsUnconfirmed.get(day.getDatum()));
            Integer ordC = PortalUtils.notNullInt(ordersConfirmed.get(day.getDatum()));
            Integer ordU = PortalUtils.notNullInt(ordersUnconfirmed.get(day.getDatum()));
            day.setCountPoptavkyZhotoveni(demC + demU);
            day.setCountObjednavkyZhotoveni(ordC + ordU);
            demandUnconfirmed += demU;
            orderUnconfirmed += ordU;
        }
    }

    private Date getFirstMonday(Date date) {

        Calendar cal = Calendar.getInstance();
        cal.setTime(date);
        //prvni den v tydnu pondeli/nedele
        int firstDayOfWeek = cal.getFirstDayOfWeek();
        cal.set(Calendar.DAY_OF_WEEK, firstDayOfWeek);

        return cal.getTime();
    }

    private Date getLastSunday(Date date) {
        Calendar cal = Calendar.getInstance();
        cal.setTime(date);
        //posledni den v tydnu nedele/sobota
        int lastDayOfWeek = cal.getFirstDayOfWeek() + 6;
        cal.set(Calendar.DAY_OF_WEEK, lastDayOfWeek);

        return cal.getTime();
    }
    
    private int getMonthCount(Date datumOd, Date datumDo) {
        Calendar start = Calendar.getInstance();
        Calendar end = Calendar.getInstance();
        start.setTime(datumOd);
        end.setTime(datumDo);
        end.add(Calendar.DAY_OF_YEAR, 1);//prictu jeden den protoze mam posledni den mesice
        
        return (end.get(Calendar.YEAR) - start.get(Calendar.YEAR)) * 12 + end.get(Calendar.MONTH) - start.get(Calendar.MONTH);
    }

    
    /*GETTERS*/
    public int getDaysInWeek() {
        return WEEK;
    }

    public int getRowCount() {
        return rowCount;
    }

    public int getColCount() {
        return colCount;
    }

    public int getMonthCount() {
        return monthCount;
    }

    public List<CalendarDay> getDays() {
        return days;
    }

    public int getDemandUnconfirmed() {
        return demandUnconfirmed;
    }

    public int getOrderUnconfirmed() {
        return orderUnconfirmed;
    }
}
