/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package cz.soma.portal.kooperace.utils;

import cz.soma.infosys.components.SDatabase;
import cz.soma.infosys.reporting.SRptConst;
import cz.soma.infosys.reporting.SRptSynchronousReportPrinter;
import cz.soma.infosys.reporting.agenda.SReportPrinter;
import cz.soma.infosys.utils.InfosysUtils;
import cz.soma.portal.kooperace.controller.PdfController;
import java.awt.GraphicsEnvironment;
import java.io.BufferedInputStream;
import java.io.DataOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.UnsupportedEncodingException;
import java.net.URL;
import java.net.URLConnection;
import java.net.URLEncoder;
import java.util.List;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.servlet.http.HttpServletResponse;
import org.openide.util.Exceptions;
import org.springframework.util.FileCopyUtils;

/**
 *
 * @author berant
 */
public class DataUtils {

    private final static String ARCHIV_VYKRES_PATH = "/cad/psarchiv/";
    private final static String ARCHIV_DATA_PATH = "/cad/psdata/";
    private final static String WWW_SERVER = "172.25.32.12";

    /**
     * Metoda pomoci www sluzby ziska pozadovana data k vykresum z adresare dat
     *
     * @param soubory seznam nazvu pozadovanych souboru
     * @param dir adresar kam se nakopiruje vysledny ZIP archiv
     * @param user nazev uzivatele, slouzi pouze pro logovani stahovanych dat
     * @return nazev ZIP archivu obsahujici vsechny pozadovana data, null =
     * neuspech
     */
    public static File getDataVykresZArchivu(List<String> soubory, File dir, String user) {
        String cesta = "";
        File zipFile = null;
        //pomoci www sluzby nactu data dle cesty k datum a nazvu souboru
        StringBuilder output = new StringBuilder();
        StringBuilder b = new StringBuilder();

        try {
            for (String nazevSouboru : soubory) {
                String cgi = String.format("http://%s/cgi-bin/is/dataVykresu.cgi", WWW_SERVER);
                String eFile = "&file=" + URLEncoder.encode(ARCHIV_DATA_PATH + nazevSouboru, "UTF-8");
                String eUser = "&user=" + URLEncoder.encode(user, "UTF-8");

                URL CGIurl = new URL(cgi);
                URLConnection c = CGIurl.openConnection();
                c.setReadTimeout(0);
                c.setDoOutput(true);
                c.setUseCaches(false);
                c.setRequestProperty("content-type", "application/x-www-form-urlencoded");

                DataOutputStream out = new DataOutputStream(c.getOutputStream());
                out.writeBytes(eFile);
                out.writeBytes(eUser);
                out.flush();
                out.close();
                File soubor = new File(dir + "/" + nazevSouboru);
                saveStreamToFile(c.getInputStream(), soubor);
            }
            zipFile = InfosysUtils.zipDir(dir);
            InfosysUtils.deleteDir(dir);

        } catch (UnsupportedEncodingException ex) {
            Exceptions.printStackTrace(ex);
        } catch (IOException ex) {
        }

        return zipFile;
    }
    
    /**
     * Metoda ulozi datovy stream do predaneho souboru
     * @param is vstupni InputStream
     * @param file soubor pro ulozeni stream
     * @throws IOException 
     */
    public static void saveStreamToFile(InputStream is, File file) throws IOException {
        if (!file.exists()) {
            file.createNewFile();
        }
        FileOutputStream fos = new FileOutputStream(file);
        BufferedInputStream bis = new BufferedInputStream(is);
        byte[] buffer = new byte[65535];
        int numberBytes = -1;
        try {
            while ((numberBytes = bis.read(buffer)) != -1) {
                fos.write(buffer, 0, numberBytes);
            }
        } catch (IOException ex) {
            Logger.getLogger(DataUtils.class.getName()).log(Level.SEVERE, null, ex);
        } finally {

            fos.flush();
            fos.close();
            bis.close();
            is.close();
        }
    }

    /**
     * Metoda pomoci www sluzby vykopiruje vykres z archivu do pozadovaneho
     * souboru
     *
     * @param archivVykresu nazev vykresu v archivu
     * @param soubor nazev souboru v archivu
     * @param user uzivatel pro ucely logovani stahovani souboru
     * @return uspech nebo neuspech
     */
    public static boolean getPDFVykresZArchivu(String archivVykresu, File soubor, String user) {

        boolean retVal = false;
        StringBuilder output = new StringBuilder();
        StringBuilder b = new StringBuilder();

        try {
            String cgi = String.format("http://%s/cgi-bin/is/ps2pdf.cgi", WWW_SERVER);
            String eFile = "&file=" + URLEncoder.encode(ARCHIV_VYKRES_PATH + archivVykresu, "UTF-8");
            String eUser = "&user=" + URLEncoder.encode(user, "UTF-8");

            URL CGIurl = new URL(cgi);
            URLConnection c = CGIurl.openConnection();
            c.setReadTimeout(0);
            c.setDoOutput(true);
            c.setUseCaches(false);
            c.setRequestProperty("content-type", "application/x-www-form-urlencoded");

            DataOutputStream out = new DataOutputStream(c.getOutputStream());
            out.writeBytes(eFile);
            out.writeBytes(eUser);
            out.flush();
            out.close();
            InfosysUtils.saveStreamToFile(c.getInputStream(), soubor);
            retVal = true;
        } catch (UnsupportedEncodingException ex) {
            Logger.getLogger(DataUtils.class.getName()).log(Level.SEVERE, null, ex);
        } catch (IOException ex) {
        }

        return retVal;
    }

    /**
     * Metoda stahne z interniho uloziste vykres a vlozi ho outputstream do predane response
     * @param archivVykres nazev souboru dle archivu vykresu
     * @param userId 
     * @param response 
     */
    public static void downloadVykres(String archivVykres, String userId, HttpServletResponse response) {
        OutputStream o = null;
        InputStream is = null;
        File file = null;
        StringBuffer errorBuffer = new StringBuffer();
        try {
            File dir = new File(String.format("/tmp/%s/", userId));
            dir.delete();
            dir.mkdir();
            dir.deleteOnExit();

            file = new File(String.format("%s/%s.pdf", dir.getAbsolutePath(), archivVykres));

            DataUtils.getPDFVykresZArchivu(archivVykres, file, "kooperacni-portal");

            response.setContentType("application/pdf");
            o = response.getOutputStream();
            is = new FileInputStream(file);
            FileCopyUtils.copy(is, o);
            response.flushBuffer();
        } catch (IOException ex) {
            Exceptions.printStackTrace(ex);
        } finally {
            try {
                o.close();
                is.close();
            } catch (IOException ex) {
                Logger.getLogger(PdfController.class.getName()).log(Level.SEVERE, null, ex);
            }
            if (file != null) {
                file.delete();
            }
        }
    }

    /**
     * Metoda vytvori tiskovou sestavu a preda ji jako outpustream do predaneho response
     * @param reportName nazev tiskove sestavy
     * @param defaultParams parametry pro vypocet tiskove sestavy
     * @param pdfName nazev vysledneho souboru
     * @param response http odpoved kam se soubor prilozi
     * @param database reference na databazi ze ktere se vytvori report
     */
    public static void downloadReport(String reportName, Map<String, Object> defaultParams, String pdfName, HttpServletResponse response, SDatabase database) {
        SRptSynchronousReportPrinter rp = new SRptSynchronousReportPrinter();

        OutputStream o = null;
        InputStream is = null;
        File file = null;
        try {
            File dir = new File("/tmp/obj/");
            dir.delete();
            dir.mkdir();
            dir.deleteOnExit();
            file = new File(String.format("%s/%s.pdf", dir.getAbsolutePath(), pdfName));
            file.deleteOnExit();
            file.canWrite();
            file.createNewFile();
            file.canRead();

            response.setContentType("application/pdf");
            database.setActive(true);
            SDatabase.setCustomDatabase(database);
            registerFonts();
            rp.exportReport(reportName, defaultParams, null, SReportPrinter.EXPORT_TYPE.PDF, file, true);
            database.setActive(false);
            o = response.getOutputStream();
            is = new FileInputStream(file);
            FileCopyUtils.copy(is, o);

            response.flushBuffer();

        } catch (IOException ex) {
            Exceptions.printStackTrace(ex);
        } finally {

            try {
                if (o != null) {
                    o.close();
                }
            } catch (IOException ex) {
                Exceptions.printStackTrace(ex);
            }

            try {
                if (is != null) {
                    is.close();
                }
            } catch (IOException ex) {
                Logger.getLogger(PdfController.class.getName()).log(Level.SEVERE, null, ex);
            }
            if (file != null) {
                file.delete();
            }
        }
    }

    /**
     * Metoda registruje pouzite fonty v sestavach do kontextu aplikace
     */
    private static void registerFonts() {
        //zjistime jstli uz nejsou fonty nahrane....
        String[] availFontFamilies = GraphicsEnvironment.getLocalGraphicsEnvironment().getAvailableFontFamilyNames();
        boolean alreadyLoaded = false;
        for (String fontFamilyName : availFontFamilies) {
            if (fontFamilyName.startsWith(SRptConst.FONT_IS_PREFIX)) {
                alreadyLoaded = true;
                break;
            }
        }
        if (alreadyLoaded) {
            return;
        }
        /*
         //jinak je nahrajem.
         for (String fontFileName : SRptConst.FONT_FILE_NAMES.values()) {
         try {
         SRptClassLoader classLoader = SRptClassLoader.getInstance();
         URL fileURL = classLoader.getResource(fontFileName);
         if (fileURL != null) {
         File f = new File(fileURL.getFile());
         Font fnt = Font.createFont(Font.TRUETYPE_FONT, f);
         GraphicsEnvironment.getLocalGraphicsEnvironment().registerFont(fnt);
         }
         } catch (IOException iOException) {
         Exceptions.printStackTrace(iOException);
         } catch (FontFormatException fontFormatException) {
         Exceptions.printStackTrace(fontFormatException);
         }
         }
         */
    }
}
