/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package cz.soma.portal.kooperace.utils;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.UnsupportedEncodingException;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.text.Format;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import org.hibernate.Session;
import org.hibernate.Transaction;
import org.openide.util.Exceptions;
import org.springframework.context.i18n.LocaleContextHolder;

/**
 *
 * @author berant
 */
public class PortalUtils {

    /**
     * Metoda pro bezpecne volani rollback Transaction
     *
     * @param trans
     */
    public static void rollbackTransaction(Transaction trans) {
        if (trans != null) {
            trans.rollback();
        }
    }

    /**
     * Metoda pro bezpecne volani commit Transaction
     *
     * @param trans
     */
    public static void commitTransaction(Transaction trans) {
        if (trans != null) {
            trans.commit();
        }
    }

    /**
     * Metoda pro bezpecne uzavreni vstupni Session
     *
     * @param session
     */
    public static void closeSession(Session session) {
        if (session != null) {
            session.close();
        }
    }

    /**
     * Metoda pro bezpecne uzavreni ResultSet
     * @param rs 
     */
    public static void closeSqlQuery(ResultSet rs) {
        closeSqlQuery(rs, true);
    }

    /**
     * Metoda pro bezpecne uzavreni ResultSet, moznost soucasne uzavrit statement
     * @param rs
     * @param closeStatement 
     */
    public static void closeSqlQuery(ResultSet rs, boolean closeStatement) {
        Statement stmt = null;
        try {
            stmt = rs != null ? rs.getStatement() : null;
        } catch (Exception ex) {
            ex.printStackTrace();
        }

        if (rs != null) {
            try {
                rs.close();
            } catch (Exception ex) {
                ex.printStackTrace();
            }
            rs = null;
        }
        if (closeStatement) {
            closeSqlStatement(stmt);
        }
    }

    /**
     * Metoda pro bezpecne uzavreni Statement
     * @param stmt 
     */
    public static void closeSqlStatement(Statement stmt) {
        if (stmt != null) {
            try {
                stmt.close();
            } catch (SQLException ex) {
            }
            stmt = null;
        }
    }

    /**
     * Vrátí řetězec obklopený apostrofy pro použití v SQL..
     *
     * @param value vstupni retezec k oapostrofovani
     * @return vstupni retezec s uvozovkami na zacatku i na konci
     */
    public static String getQuotedStr(String value) {
        return "'" + value + "'"; //NOI18N
    }

    /**
     * Metoda prevede textovy retezec na datum
     *
     * @param strDate vstupni retezec s datem ve formatu d.M.y
     * @return Date ze zadaneho retezce
     * @throws ParseException
     */
    public static Date strToDate(String strDate) throws ParseException {
        return strToDate(strDate, "d.M.y"); //NOI18N
    }

    /**
     * Metoda prevede textovy retezec na datum
     *
     * @param strDate vstupni retezec s datem
     * @param dateFormat format data ve stupnim retezci
     * @return Date ze zadaneho retezce
     * @throws ParseException
     */
    public static Date strToDate(String strDate, String dateFormat) throws ParseException {
        Date retVal = null;
        //DateFormat f = DateFormat.getInstance();
        SimpleDateFormat f = new SimpleDateFormat(dateFormat);
        retVal = f.parse(strDate);
        return retVal;
    }

    /**
     * Vrátí naformátované datum pro použití v SQL.
     *
     * @param date
     * @return
     */
    public static String getParametrizedDate(Date date) {
        return getParametrizedDate(date, true);
    }

    /**
     * Vrátí naformátované datum s volbou uzavreni do uvozovek
     * @param date
     * @param quoted
     * @return 
     */
    public static String getParametrizedDate(Date date, boolean quoted) {
        return getParametrizedDate(date, quoted, false);
    }

    /**
     * Vrátí naformátované datum s volbou uvozovek a zobrazeni casu
     * @param date
     * @param quoted
     * @param showTime
     * @return 
     */
    public static String getParametrizedDate(Date date, boolean quoted, boolean showTime) {
        Format formatter = new SimpleDateFormat("d.M.yyyy" + (showTime ? " H:m:s.S" : "")); //NOI18N
        return date == null ? "null" : //NOI18N
                (quoted ? "'" : "") + formatter.format(date) + (quoted ? "'" : ""); //NOI18N
    }

    /**
     * Metoda pro bezpecny prevod Integer na String
     *
     * @param i vstupni integer
     * @return prevedeny String, pro null vraci 0
     */
    public static String intToStr(Integer i) {
        if (i == null) {
            return "0"; //NOI18N
        }
        return i.toString();
    }

    /**
     * Metoda string, pri zadani null hodnoty vraci prazdny retezec
     *
     * @param s
     * @return
     */
    public static String notNullString(String s) {
        if (s == null) {
            return "";
        }
        return s;
    }

    /**
     * Metoda vraci nenull hodnotu vstupniho parametru, ktery muze byt null
     *
     * @param i
     * @return
     */
    public static Integer notNullInt(Integer i) {
        return i == null ? 0 : i;
    }

    /**
     * Metoda vraci prvni den v mesici
     *
     * @param month pozadovany mesic
     * @param year pozadovany rok
     * @return Date prvni den v mesici
     * @throws ParseException
     */
    public static Date firstDayOfMonth(int month, int year) throws ParseException {
        String date = "1." + month + "." + year;
        return strToDate(date, "d.M.yyyy");
    }

    /**
     * Metoda vraci posledni den v mesici
     *
     * @param month pozadovany mesic
     * @param year pozadovany rok
     * @return Date posledni den v mesici
     * @throws ParseException
     */
    public static Date lastDayOfMonth(int month, int year) throws ParseException {
        //zjistim si prvni den
        Calendar cal = Calendar.getInstance();
        cal.setTime(firstDayOfMonth(month, year));
        //pridam jeden mesic - ziskam nasledujici mesic
        cal.add(Calendar.MONTH, 1);
        //od nasledujiciho mesice odectu jeden den a ziskam hledany posledni den
        cal.add(Calendar.DAY_OF_YEAR, -1);

        return cal.getTime();
    }

    /**
     * Z aktualniho data ziska pozadovanou hodnotu
     *
     * @param calendarField int reprezentujici pozadovanou hodnotu (ex:
     * Calendar.DAY_OF_YEAR)
     * @param addMonth prida k aktualnimu datu jeden mesic
     * @return pozadovana hodnota
     */
    public static int getCurrentDayField(int calendarField, boolean addMonth) {
        Calendar cal = Calendar.getInstance();
        if (addMonth) {
            cal.add(Calendar.MONTH, 1);
        }
        int val = cal.get(calendarField);
        return val;
    }

    /**
     * Z aktualniho data ziska pozadovanou hodnotu
     * @param calendarField
     * @return 
     */
    public static int getCurrentDayField(int calendarField) {
        return getCurrentDayField(calendarField, false);
    }

    /**
     * Vraci dnesni datum
     * @return 
     */
    public static Date getCurrentDate() {
        Calendar cal = Calendar.getInstance();
        return cal.getTime();
    }

    /**
     * Metoda otestuje vstupni mesic a rok ze je validni (tzn. mesic 1-12 a rok 2011 - 2020)
     * @param month
     * @param year
     * @return 
     */
    public static boolean isValidMonthAndYearValues(Integer month, Integer year) {
        if (month == null || year == null || month > 12 || month < 1) {
            return false;
        }
        if (year < 2011 || year > 2020) {
            return false;
        }

        return true;
    }

    /**
     * Prevod textu na cislo
     * @param value
     * @return 
     */
    public static Integer strToIntDef(String value) {
        return strToIntDef(value, 0);
    }

    /**
     * Prevod textu na cislo s default hodnotou
     * @param value
     * @param defaultValue
     * @return 
     */
    public static Integer strToIntDef(String value, Integer defaultValue) {
        Integer retVal = defaultValue;
        if (value != null) {
            try {
                retVal = Integer.valueOf(value.trim());
            } catch (NumberFormatException ex) {
                //ignored --vracime defaultValue
            }
        }
        return retVal;
    }

    /**
     * Vraci lokalizaci Reportu dle lokalizace uzivatele
     *
     * @return koncovka nazvu reportu (CZ, EN prip RU)
     */
    public static String getDefaultLocaleOfReport() {
        return LocaleContextHolder.getLocale().getLanguage().equals("cs") ? "CZ" : "EN";
    }

    /**
     * Metoda pro prevod na spravne kodovani cestiny do database
     *
     * @param text
     * @return
     */
    public static String encodeString(String text) {
        String enc = "";
        try {
            enc = new String(text.getBytes("ISO-8859-1"));
        } catch (UnsupportedEncodingException ex) {
            Exceptions.printStackTrace(ex);
        }
        return enc;
    }

    /**
     * Vraci textovy vystup reprezentujici html novinek stahuje se z externiho
     * zdroje
     * Metoda pro stazeni http novinek na https strankach
     * @return
     */
    public static String getNewsData() {

        BufferedReader in = null;
        StringBuffer result = new StringBuffer();
        try {
            URL newsUrl = new URL("http://www.soma-eng.com/i_news?lang=sc");
            URLConnection yc = newsUrl.openConnection();
            in = new BufferedReader(new InputStreamReader(
                    yc.getInputStream()));
            String inputLine;
            while ((inputLine = in.readLine()) != null) {
                result.append(inputLine);
            }
        } catch (MalformedURLException ex) {
            Exceptions.printStackTrace(ex);
            return "";
        } catch (IOException ex) {
            Exceptions.printStackTrace(ex);
            return "";
        } finally {
            if (in != null) {
                try {
                    in.close();
                } catch (IOException ex) {

                }
            }
        }

        //nahrazeni spravnymi cestami
        String res = result.toString();
        res = res.replace("/www/i_news/js_i/", "./js/");
        res = res.replace("/www/i_news/css_i/", "./css/");
        res = res.replace("/cs/", "http://www.soma-eng.com/cs/");
        res = res.replace("/en/", "http://www.soma-eng.com/en/");

        return res;
    }
}
